@@ -12,14 +12,20 @@ class Field:
         alias: str = None,
         freeze: bool = False,
         exclude: bool = None,
+        validator: Callable = None,
     ) -> None:
         self.default = default
         self.alias = alias
         self.freeze = freeze
         self.annotation = None
         self.exclude = exclude
+        self.validator = validator
+
+
+def get_fields_info(cls: Any) -> Dict[str, Field]:
+    return _get_fields_info(cls)
 
 
 @lru_cache(typed=True)
-def get_fields_info(cls: Any):
+def _get_fields_info(cls: Any) -> Dict[str, Field]:
     return {}
