@@ -1,6 +1,8 @@
 import inspect
 from typing import *
 
+from .field import Field, get_fields_info
+from .dataclass import Dataclass, Validator
 from .types import Generic, get_args, get_origin
 
 __all__ = ["validator", "Validator", "validate"]
@@ -25,28 +27,31 @@ def validator(*types: Type[Generic], func: Callable = None):
     return decorator
 
 
-def validate(G: Union[Generic, Type], obj: Any, strict: bool = True):
+def validate(G: Union[Generic, Type], obj: Any, **kwargs):
     if isinstance(G, get_args(Generic)):
         origin = get_origin(G) if G is not Any else Any
-
-        if origin not in validators:
-            return not strict
-
+        assert origin in validators, f"Validator not found"
         validator = validators[origin]
         return validator(G, obj)
 
+    if issubclass(G, Dataclass):
+        validator = validators[Dataclass]
+        return validator(G, obj, **kwargs)
+
     return isinstance(obj, G)
 
 
 @validator(List, Set, Sequence)
-def validate_sequence(G: Generic, obj: Any):
+def validate_sequence(G: Generic, obj: Any, **kwargs):
     arg, *_ = get_args(G)
     origin = get_origin(G)
-    return issubclass(type(obj), origin) and all(validate(arg, elem) for elem in obj)
+    return issubclass(type(obj), origin) and all(
+        validate(arg, elem, **kwargs) for elem in obj
+    )
 
 
 @validator(Tuple)
-def validate_tuple(G: Generic, obj: Any):
+def validate_tuple(G: Generic, obj: Any, **kwargs):
     args = get_args(G)
     if not issubclass(type(obj), tuple):
         return False
@@ -56,7 +61,7 @@ def validate_tuple(G: Generic, obj: Any):
 
     curr_idx = 0
     for i, elem in enumerate(obj):
-        if not validate(args[curr_idx], elem):
+        if not validate(args[curr_idx], elem, **kwargs):
             return False
 
         if i < len(obj) - 1:
@@ -71,46 +76,46 @@ def validate_tuple(G: Generic, obj: Any):
 
 
 @validator(Dict)
-def validate_dict(G: Generic, obj: Any):
+def validate_dict(G: Generic, obj: Any, **kwargs):
     if not issubclass(type(obj), dict):
         return False
 
     key_type, val_type = get_args(G)
-    is_key_valid = all(validate(key_type, key) for key in obj.keys())
-    is_val_valid = all(validate(val_type, val) for val in obj.values())
+    is_key_valid = all(validate(key_type, key, **kwargs) for key in obj.keys())
+    is_val_valid = all(validate(val_type, val, **kwargs) for val in obj.values())
 
     return is_key_valid and is_val_valid
 
 
 @validator(Union)
-def validate_union(G: Generic, obj: Any):
+def validate_union(G: Generic, obj: Any, **kwargs):
     args = get_args(G)
-    return any(validate(arg, obj) for arg in args)
+    return any(validate(arg, obj, **kwargs) for arg in args)
 
 
 @validator(type(None))
-def validate_none(G: Generic, obj: Any):
+def validate_none(G: Generic, obj: Any, **kwargs):
     return obj is None
 
 
 @validator(Any)
-def validate_any(G: Generic, obj: Any):
+def validate_any(G: Generic, obj: Any, **kwargs):
     return True
 
 
 @validator(Hashable)
-def validate_hashable(G: Generic, obj: Any):
+def validate_hashable(G: Generic, obj: Any, **kwargs):
     return hasattr(obj, "__hash__")
 
 
 @validator(Type)
-def validate_type(G: Generic, obj: Any):
+def validate_type(G: Generic, obj: Any, **kwargs):
     arg, *_ = get_args(G)
     return arg == obj
 
 
 @validator(Callable)
-def validate_callable(G: Generic, obj: Any):
+def validate_callable(G: Generic, obj: Any, **kwargs):
     if not hasattr(obj, "__call__"):
         return False
 
@@ -125,12 +130,12 @@ def validate_callable(G: Generic, obj: Any):
     for input_type in inputs_type:
         param = next(params)
         if param.annotation is not inspect._empty and not validate(
-            Type[input_type], param.annotation
+            Type[input_type], param.annotation, **kwargs
         ):
             return False
 
     if signature.return_annotation is not inspect._empty and not validate(
-        Type[output_type], signature.return_annotation
+        Type[output_type], signature.return_annotation, **kwargs
     ):
         return False
 
@@ -138,14 +143,54 @@ def validate_callable(G: Generic, obj: Any):
 
 
 @validator(Iterable, Iterator)
-def validate_iterable(G: Generic, obj: Any):
+def validate_iterable(G: Generic, obj: Any, **kwargs):
     return isinstance(obj, Iterable)
 
 
-class Validator:
-    def __init__(self, field: str, func: Callable):
-        self.field = field
-        self.func = func
+@validator(Dataclass)
+def validate_dataclass(
+    G: Dataclass, obj: Any, as_schema: bool = False, strict: bool = False, **kwargs
+):
+    from .parser import asclass
+    if not as_schema:
+        return isinstance(obj, G)
+
+    if not isinstance(obj, Mapping):
+        return False
+
+    fields = get_fields_info(G)
+    fields = {
+        field.alias if field.alias is not None else key: field
+        for key, field in fields.items()
+    }
+
+    for key, val in obj.items():
+        if key not in fields:
+            if strict:
+                return False
+            continue
+
+        field: Field = fields.pop(key)
+        if not validate(
+            field.annotation, val, as_schema=as_schema, strict=strict, **kwargs
+        ):
+            return False
+        
+        if field.validator is not None:
+            if issubclass(field.annotation, Dataclass):
+                try:
+                    val = asclass(field.annotation, val)
+                except AssertionError:
+                    return False
+                
+                if not field.validator(val):
+                    return False
+
+    for key, field in fields.items():
+        if field.default == Ellipsis:
+            return False
+
+    return True
 
 
 def validator(field: str, func: Callable = None):
